#include <glib-object.h>
#include <string.h>
#include <stdio.h>

#include "widget.h"
#include "xml.h"
#include "page.h"
#include "cb.h"
#include "group.h"
#include "prop-util.h"
#include "tag.h"

#define WPS_WIDGET_GET_PRIVATE(obj)(	G_TYPE_INSTANCE_GET_PRIVATE ((obj),	\
					WPS_TYPE_WIDGET,			\
					WPSWidgetPrivate))

struct _WPSWidgetPrivate {
	gint tmp;
};

enum {
	UPDATE_PROP,
	LAST_SIGNAL
};

enum {
	PROP_WIDGET_EMPTY,
	PROP_WIDGET_NAME,
	PROP_WIDGET_TYPE,
	PROP_WIDGET_INFO
};


static guint widget_signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE(WPSWidget, wps_widget, G_TYPE_OBJECT);


void wps_widget_create_gvalue(LinkAttributeInfo *link_attr_info, GValue *value)
{
	if (!g_strcmp0(link_attr_info->type->str, "string"))
		g_value_init(value, G_TYPE_STRING);
	else if (!g_strcmp0(link_attr_info->type->str, "int"))
		g_value_init(value, G_TYPE_INT);
	else if (!g_strcmp0(link_attr_info->type->str, "uint"))
		g_value_init(value, G_TYPE_UINT);
	else if (!g_strcmp0(link_attr_info->type->str, "float"))
		g_value_init(value, G_TYPE_FLOAT);
	else if (!g_strcmp0(link_attr_info->type->str, "double"))
		g_value_init(value, G_TYPE_DOUBLE);
	else if (!g_strcmp0(link_attr_info->type->str, "long"))
		g_value_init(value, G_TYPE_LONG);
	else if (!g_strcmp0(link_attr_info->type->str, "bool")) {
		g_value_init(value, G_TYPE_BOOLEAN);
		g_value_set_boolean(value, FALSE);
	}
	else
		g_warning("Unknown data type in LinkAttributInfo %s\n", link_attr_info->type->str);
}

static GObject *wps_widget_constructor(GType type,
					guint n_construct_properties,
					GObjectConstructParam *construct_properties)
{
	GObject *obj;
	WPSWidget *widget;
	GParameter *gparam;
	GObjectClass *object_class;
	GList *last_signal = NULL;
	LinkAttributeInfo *link_attr_info;
	GList *last_link_attr_info = NULL;
		
	obj = G_OBJECT_CLASS(wps_widget_parent_class)->constructor(type, n_construct_properties, construct_properties);
	widget = WPS_WIDGET(obj);

	object_class = g_type_class_ref(wps_get_type_from_name(widget->widget_info->type_name->str));
	gparam = set_gparam(object_class,
			widget->widget_info->prop_info_array,
			widget->widget_info->prop_len);
	/* create object */
	widget->object = g_object_newv(	wps_get_type_from_name(widget->widget_info->type_name->str),
					widget->widget_info->prop_len,
					gparam);	
	/* signals */	
	for (last_signal = g_list_last(widget->widget_info->signal_info); last_signal != NULL; last_signal = g_list_previous(last_signal)) {
		SignalInfo *signal = last_signal->data;
				
		add_widget_to_hash(signal->handler->str, widget);
	}
	
	/* style */
	if (widget->widget_info->style_info != NULL && GTK_IS_WIDGET(widget->object)) {
		GtkWidget *gtk_widget = (GtkWidget *)widget->object;
		GtkWidget *child;
		GList *children_list = NULL;
		GList *last_child = NULL;
		GdkColor color;
		
		if (GTK_IS_CONTAINER(gtk_widget)) {
			children_list = gtk_container_get_children((GtkContainer *)gtk_widget);
			if (children_list != NULL) {
				for (last_child = g_list_last(children_list); last_child != NULL; last_child = g_list_previous(last_child)) {
					child = last_child->data;
					
					if (widget->widget_info->style_info->font_size != 0)
						widget_pango_font_set(child, widget->widget_info->style_info);
					if (widget->widget_info->style_info->field_color != NULL) {
						gdk_color_parse (widget->widget_info->style_info->field_color, &color);
						gtk_widget_modify_base((GtkWidget *)child, GTK_STATE_NORMAL, &color);
					}
					if (widget->widget_info->style_info->field_font_color != NULL) {
						gdk_color_parse (widget->widget_info->style_info->field_font_color, &color);
						gtk_widget_modify_text((GtkWidget *)child, GTK_STATE_NORMAL, &color);
					}
				}
			}
		}
		//widget_pango_font_set(gtk_widget, widget->widget_info->style_info);
		//gtk_widget_modify_base((GtkWidget *)widget->object, GTK_STATE_NORMAL, &color);
		//gtk_widget_modify_text((GtkWidget *)widget->object, GTK_STATE_NORMAL, &color);
		if (widget->widget_info->style_info->font_size != 0)
			widget_pango_font_set(gtk_widget, widget->widget_info->style_info);
		if (widget->widget_info->style_info->field_color != NULL) {
			gdk_color_parse (widget->widget_info->style_info->field_color, &color);
			gtk_widget_modify_base((GtkWidget *)widget->object, GTK_STATE_NORMAL, &color);
		}
		if (widget->widget_info->style_info->field_font_color != NULL) {
			gdk_color_parse (widget->widget_info->style_info->field_font_color, &color);
			gtk_widget_modify_text((GtkWidget *)widget->object, GTK_STATE_NORMAL, &color);
		}
	}
	
	/* link attributes */
	
	if (widget->widget_info->link_attributes_info != NULL) {
		for (last_link_attr_info = g_list_last(widget->widget_info->link_attributes_info); last_link_attr_info != NULL; last_link_attr_info = g_list_previous(last_link_attr_info)) {
			LinkAttribute *link_attr;
			GValue *value;
			Tag *tag;
			
			link_attr_info = last_link_attr_info->data;
			link_attr = g_new0(LinkAttribute, 1);
			link_attr->name = g_strdup(link_attr_info->name->str);
			link_attr->tag = g_strdup(link_attr_info->tag->str);
			
			if (!g_strcmp0(link_attr_info->direction->str, "DIR_IN"))
				link_attr->direction = DIR_IN;
			else if (!g_strcmp0(link_attr_info->direction->str, "DIR_OUT"))
				link_attr->direction = DIR_OUT;
			else 
				g_warning("Unknown direction type\n");
			
			value = g_new0(GValue, 1);
			wps_widget_create_gvalue(link_attr_info, value);
			link_attr->value = value;
						
			tag_table_add_empty_key("test");
			tag = tag_table_find_tag_by_name(link_attr->tag);
			
			if (tag != NULL)
				link_attr->connected_tag = tag;
				
			widget->link_attributes = g_list_append(widget->link_attributes, link_attr);
		}
	}
	
	
	
	
	return obj;
}

static void wps_widget_init(WPSWidget *widget)
{
	widget->name = NULL;
	widget->link_attributes = NULL;
}


static void wps_widget_get_property(	GObject *object,
					guint property_id,
					GValue *value,
					GParamSpec *pspec)
{
	WPSWidget *widget = (WPSWidget *)object;
	
	switch (property_id) {
		/*case PROP_WIDGET_NAME:
			g_value_set_string(value, widget->name );
			break;
		case PROP_WIDGET_TYPE:
			g_value_set_string(value, widget->type_name);
			break;*/
		case PROP_WIDGET_INFO:
			g_value_set_pointer(value, widget->widget_info);
			break;
		default:
			break;
	}
}

static void wps_widget_set_property(	GObject *object,
					guint property_id,
					const GValue *value,
					GParamSpec *pspec)
{
	WPSWidget *widget = (WPSWidget *)object;
	
	switch (property_id) {
		/*case PROP_WIDGET_NAME:
			widget->name = g_value_dup_string(value);
			break;
		case PROP_WIDGET_TYPE:
			widget->type_name = g_value_dup_string(value);
			break;*/
		case PROP_WIDGET_INFO:
			widget->widget_info = g_value_get_pointer(value);
			break;		
		default:
			break;
	}
}

void test_cb(WPSWidget *widget)
{
	g_warning("=====================TEST========================\n");
}

static void wps_widget_class_init(WPSWidgetClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS(klass);	
	object_class->constructor = wps_widget_constructor;
	object_class->get_property = wps_widget_get_property;
	object_class->set_property = wps_widget_set_property; 
	
	klass->update_prop = NULL;
	/*g_object_class_install_property(object_class,
					PROP_WIDGET_NAME,
					g_param_spec_string("name",
					"Name",
					"Common name of widget",
					NULL,
					G_PARAM_READWRITE |
				      	G_PARAM_CONSTRUCT));
	
	g_object_class_install_property(object_class,
					PROP_WIDGET_TYPE,
					g_param_spec_string("type",
					"Type",
					"Gtk Widget Type",
					NULL,
					G_PARAM_READWRITE |
				      	G_PARAM_CONSTRUCT));
				      				*/
	g_object_class_install_property(object_class,
					PROP_WIDGET_INFO,
					g_param_spec_pointer("widget_info", 
					"WidgetInfo",
					"Widget Info Structute",
					G_PARAM_READWRITE |
					G_PARAM_CONSTRUCT));
	
	widget_signals[UPDATE_PROP] = g_signal_new ("update_prop",
						G_OBJECT_CLASS_TYPE (object_class),
						G_SIGNAL_RUN_FIRST,
						G_STRUCT_OFFSET (WPSWidgetClass, update_prop),
						NULL, NULL,
						g_cclosure_marshal_VOID__VOID,
						G_TYPE_NONE, 0, NULL);
	
	g_type_class_add_private (klass, sizeof (WPSWidgetPrivate));
}

GList *wps_widget_create_from_info(GObject *container, GList *widget_info)
{
	GList *widget_list = NULL;
	GList *last_widget;
	
	for (last_widget = g_list_last(widget_info); last_widget != NULL; last_widget = g_list_previous(last_widget)) {
		WPSWidget *widget;
		WidgetInfo *widget_info;
		
		widget_info = last_widget->data;
		widget = g_object_new(WPS_TYPE_WIDGET, "widget_info", widget_info, NULL);
		
		if (WPS_IS_PAGE(container))
			WPS_PAGE_GET_CLASS(container)->add_widget((WPSPage *)container, (GObject *)widget);
		else if (WPS_IS_GROUP(container))
			WPS_GROUP_GET_CLASS(container)->add_widget((WPSGroup *)container, widget);
		else
			g_warning("Unknown object for placing widget\n");
			
		widget_list = g_list_append(widget_list, widget);
	}
	
	return widget_list;
}
